/* This file was generated by scramble.py. */
#line 1 "src/world.py"
#include "world.h"
#line 4
#define W 100
#define H 100
#line 15
static Tile dummy;
static LandVector light = {- 1, - 1, 1};
static void _update(World * self, int ax, int ay, int bx, int by);
static void _update_vertex(World * self, int tr, float x, float y, Tile * t);
static LandVector _get_normal(World * self, LandFloat x, LandFloat y);
#line 18
int world_width(void) {
#line 18
    return W;
}
#line 19
int world_height(void) {
#line 19
    return H;
}
Tile* world_tile(World * world, int x, int y) {
    if (x < 0 || y < 0 || x >= W || y >= H) {
#line 22
        return & dummy;
    }
#line 23
    return world->tiles + y * W + x;
}
World* world_new(void) {
    World * self;
    land_alloc(self);
    self->tiles = land_malloc(W * H * sizeof (* self->tiles));
#line 30
    for (int y = 0; y < H; y += 1) {
        for (int x = 0; x < W; x += 1) {
            double dx = x - W / 2;
            double dy = y - H / 2;
            double d = sqrt(dx * dx + dy * dy);
            double h = 80 - d * 2;
            h += land_rnd(0, 0.5);
            if (h < 0) {
#line 37
                h = 0;
            }
#line 38
            Tile * t = world_tile(self, x, y);
            t->height = h;
            t->color = land_color_rgba(0.4, 0.2, 0, 1);
        }
    }
#line 42
    for (int i = 0; i < 40; i += 1) {
        float r = land_rnd(5, 40);
        float rr = land_rnd(0, 0.1);
        float rg = land_rnd(0, 0.1);
        float rb = land_rnd(0, 0.1);
        int rx = land_rand(0, W - 1);
        int ry = land_rand(0, H - 1);
        for (int y = 0; y < H; y += 1) {
            for (int x = 0; x < W; x += 1) {
                Tile * t = world_tile(self, x, y);
                double dx = x - rx;
                double dy = y - ry;
                double d = sqrt(dx * dx + dy * dy);
                float c = 1 - d / r;
                if (c < 0) {
#line 56
                    c = 0;
                }
#line 57
                float cr = t->color.r + c * rr;
                float cg = t->color.g + c * rg;
                float cb = t->color.b + c * rb;
                if (cr > 1) {
#line 60
                    cr = 1;
                }
#line 61
                if (cg > 1) {
#line 61
                    cg = 1;
                }
#line 62
                if (cb > 1) {
#line 62
                    cb = 1;
                }
                t->color = land_color_rgba(cr, cg, cb, 1);
            }
        }
    }
#line 66
    world_create_mesh(self);
#line 68
    _update(self, 0, 0, 100, 100);
#line 70
    return self;
}
void world_destroy(World * self) {
    land_free(self->tiles);
    mesh_destroy(self->mesh);
    land_free(self);
}
void world_blotch(World * self, LandFloat px, LandFloat py, LandFloat radius, LandColor color) {
    double s = 480;
#line 80
    int ax = (px - radius + s) * 50 / s;
    int ay = (py - radius + s) * 50 / s;
    int bx = (px + radius + s) * 50 / s;
    int by = (py + radius + s) * 50 / s;
    bx++;
    by++;
#line 87
    if (ax < 0) {
#line 87
        ax = 0;
    }
#line 88
    if (ay < 0) {
#line 88
        ay = 0;
    }
#line 89
    if (bx > W) {
#line 89
        bx = W;
    }
#line 90
    if (by > H) {
#line 90
        by = H;
    }
    int ix = (px + s) * 50 / s;
    int iy = (py + s) * 50 / s;
    int ir = radius * 50 / s;
#line 96
    for (int y = ay; y < by; y += 1) {
        for (int x = ax; x < bx; x += 1) {
#line 99
            float d = sqrt((x - ix) * (x - ix) + (y - iy) * (y - iy));
            if (d > ir) {
                continue;
            }
#line 102
            Tile * t = world_tile(self, x, y);
            float i = (ir - d) / ir;
            float r = t->color.r * (1 - i) + color.r * i;
            float g = t->color.g * (1 - i) + color.g * i;
            float b = t->color.b * (1 - i) + color.b * i;
            t->color = land_color_rgba(r, g, b, 1);
        }
    }
#line 109
    _update(self, ax, ay, bx, by);
}
void world_patch(World * self, LandFloat px, LandFloat py, LandFloat r, LandColor color) {
    double s = 480;
#line 114
    int ax = (px - r + s) * 50 / s;
    int ay = (py - r + s) * 50 / s;
    int bx = (px + r + s) * 50 / s;
    int by = (py + r + s) * 50 / s;
    bx++;
    by++;
#line 121
    if (ax < 0) {
#line 121
        ax = 0;
    }
#line 122
    if (ay < 0) {
#line 122
        ay = 0;
    }
#line 123
    if (bx > W) {
#line 123
        bx = W;
    }
#line 124
    if (by > H) {
#line 124
        by = H;
    }
    for (int y = ay + 1; y < by - 1; y += 1) {
        for (int x = ax + 1; x < bx - 1; x += 1) {
            Tile * t = world_tile(self, x, y);
            t->color = color;
        }
    }
#line 131
    _update(self, ax, ay, bx, by);
}
static void _update(World * self, int ax, int ay, int bx, int by) {
    double s = 480;
    for (int v = ay; v < by; v += 1) {
        for (int u = ax; u < bx; u += 1) {
            int tr1 = 2 * (u + v * 100);
            int tr2 = tr1 + 1;
#line 140
            Tile * t1 = world_tile(self, u, v);
            Tile * t2 = world_tile(self, u + 1, v);
            Tile * t3 = world_tile(self, u + 1, v + 1);
            Tile * t4 = world_tile(self, u, v + 1);
#line 145
            float x1 = - s + u * s * 2 / 100;
            float y1 = - s + v * s * 2 / 100;
            float x2 = - s + (u + 1) * s * 2 / 100;
            float y2 = - s + v * s * 2 / 100;
            float x3 = - s + (u + 1) * s * 2 / 100;
            float y3 = - s + (v + 1) * s * 2 / 100;
            float x4 = - s + u * s * 2 / 100;
            float y4 = - s + (v + 1) * s * 2 / 100;
#line 154
            _update_vertex(self, tr1 * 3 + 0, x1, y1, t1);
            _update_vertex(self, tr1 * 3 + 1, x2, y2, t2);
            _update_vertex(self, tr1 * 3 + 2, x3, y3, t3);
            _update_vertex(self, tr2 * 3 + 0, x3, y3, t3);
            _update_vertex(self, tr2 * 3 + 1, x4, y4, t4);
            _update_vertex(self, tr2 * 3 + 2, x1, y1, t1);
        }
    }
}
#line 161
static void _update_vertex(World * self, int tr, float x, float y, Tile * t) {
    LandVector n = _get_normal(self, x, y);
    LandFloat d = land_vector_dot(n, light);
    d = (1 + d) / 2;
    float r = t->color.r * d;
    float g = t->color.g * d;
    float b = t->color.b * d;
    mesh_update_vertex(self->mesh, tr, x, y, t->height, r, g, b, 1);
}
static LandVector _get_normal(World * self, LandFloat x, LandFloat y) {
    LandFloat s = 9.61;
    LandFloat h = world_get_altitude(self, x, y);
    LandFloat ha = world_get_altitude(self, x - s, y);
    LandFloat hb = world_get_altitude(self, x, y - s);
    LandVector a = land_vector(- s, 0, ha - h);
    LandVector b = land_vector(0, - s, hb - h);
    LandVector n = land_vector_normalize(land_vector_cross(a, b));
    return n;
}
LandFloat world_get_altitude(World * world, LandFloat x, LandFloat y) {
    LandFloat s = 9.61;
    Tile * t = world_tile(world, (int)(x + 480) / s, (int)(y + 480) / s);
    return t->height;
}
LandColor world_get_color(World * world, LandFloat x, LandFloat y) {
    LandFloat s = 9.61 /* 9.6 doesn't work? */;
    Tile * t = world_tile(world, (int)(x + 480) / s, (int)(y + 480) / s);
    return t->color;
}
void world_create_mesh(World * world) {
    if (! world->mesh) {
        world->mesh = mesh_make("world");
    }
#line 193
    land_triangles_clear(world->mesh->triangles);
#line 195
    LandCSG * csg = csg_grid(100, 100, NULL);
    land_csg_transform(csg, land_4x4_matrix_scale(480, 480, 1));
    land_csg_triangles(csg);
    mesh_add_csg(world->mesh, csg);
#line 200
    land_csg_destroy(csg);
}
/* This file was generated by scramble.py. */
